==> gromacs-2024.6.tar.gz: gromacs-2024.6/src/gromacs/trajectoryanalysis/modules/scattering-debye-saxs.cpp <==
/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright 2023- The GROMACS Authors
 * and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
 * Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * https://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at https://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out https://www.gromacs.org.
 */
/*! \internal \file
 * \brief
 * Implements class for SAXS Debye Scattering
 *
 * \author Alexey Shvetsov <alexxyum@gmail.com>
 * \ingroup module_trajectoryanalysis
 */

#include "gmxpre.h"

#include "scattering-debye-saxs.h"

#include <cmath>

#include <vector>

#include "gromacs/math/units.h"

namespace gmx
{


SaxsDebye::SaxsDebye(std::vector<Isotope> isotopes, const std::vector<double>& qList) :
    isotopes_(std::move(isotopes))
{
    sfDepenOnQ_                                             = true;
    std::vector<AtomicStructureFactor> atomicScatterFactors = readAtomicStructureFactors();
    for (auto q : qList)
    {
        for (auto scatter : atomicScatterFactors)
        {

            double scattering = scatter.xrayCromerMannParameters.c;
            double q4pi       = q / (4.0 * M_PI);
            for (int j = 0; j < 4; j++)
            {
                scattering += scatter.xrayCromerMannParameters.a[j]
                              * exp(-scatter.xrayCromerMannParameters.b[j] * q4pi * q4pi);
            }
            Isotope isotope       = getIsotopeFromString(scatter.isotope);
            auto    pair          = std::make_pair(static_cast<int>(isotope), q);
            scatterFactors_[pair] = scattering;
        }
    }
}

double SaxsDebye::getScatteringLength(int i, double q)
{
    auto   pair       = std::make_pair(static_cast<int>(isotopes_[i]), q);
    double scattering = scatterFactors_[pair];
    return scattering;
}


} // namespace gmx

==> gromacs-2026.0.tar.gz: gromacs-2026.0/src/gromacs/trajectoryanalysis/modules/scattering-debye-saxs.cpp <==
/*
 * This file is part of the GROMACS molecular simulation package.
 *
 * Copyright 2023- The GROMACS Authors
 * and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
 * Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
 *
 * GROMACS is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * GROMACS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GROMACS; if not, see
 * https://www.gnu.org/licenses, or write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * If you want to redistribute modifications to GROMACS, please
 * consider that scientific software is very special. Version
 * control is crucial - bugs must be traceable. We will be happy to
 * consider code for inclusion in the official distribution, but
 * derived work must not be called official GROMACS. Details are found
 * in the README & COPYING files - if they are missing, get the
 * official version at https://www.gromacs.org.
 *
 * To help us fund GROMACS development, we humbly ask that you cite
 * the research papers on the package. Check out https://www.gromacs.org.
 */
/*! \internal \file
 * \brief
 * Implements class for SAXS Debye Scattering
 *
 * \author Alexey Shvetsov <alexxyum@gmail.com>
 * \ingroup module_trajectoryanalysis
 */

#include "gmxpre.h"

#include "scattering-debye-saxs.h"

#include <cmath>

#include <array>
#include <vector>

#include "gromacs/math/units.h"
#include "gromacs/trajectoryanalysis/modules/isotope.h"
#include "gromacs/trajectoryanalysis/modules/scatteringfactors.h"

namespace gmx
{


SaxsDebye::SaxsDebye(std::vector<Isotope> isotopes, const std::vector<double>& qList) :
    isotopes_(std::move(isotopes))
{
    sfDepenOnQ_                                             = true;
    std::vector<AtomicStructureFactor> atomicScatterFactors = readAtomicStructureFactors();
    for (auto q : qList)
    {
        for (auto scatter : atomicScatterFactors)
        {

            double scattering = scatter.xrayCromerMannParameters.c;
            double q4pi       = q / (4.0 * M_PI);
            for (int j = 0; j < 4; j++)
            {
                scattering += scatter.xrayCromerMannParameters.a[j]
                              * exp(-scatter.xrayCromerMannParameters.b[j] * q4pi * q4pi);
            }
            Isotope isotope       = getIsotopeFromString(scatter.isotope);
            auto    pair          = std::make_pair(static_cast<int>(isotope), q);
            scatterFactors_[pair] = scattering;
        }
    }
}

double SaxsDebye::getScatteringLength(int i, double q)
{
    auto   pair       = std::make_pair(static_cast<int>(isotopes_[i]), q);
    double scattering = scatterFactors_[pair];
    return scattering;
}


} // namespace gmx
